<?php

namespace Drupal\view_entity_render_popup\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Plugin implementation of the 'view_entity_render_popup' formatter.
 *
 * @FieldFormatter(
 *   id = "view_entity_render_popup",
 *   label = @Translation("Rendered Entity Popup"),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class ViewEntityRenderPopupFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'view_mode' => 'default',
      'link_text' => 'View',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $elements['view_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('View mode'),
      '#options' => \Drupal::service('entity_display.repository')->getViewModeOptions($this->fieldDefinition->getSetting('target_type')),
      '#default_value' => $this->getSetting('view_mode'),
      '#description' => $this->t('Select the view mode to render the referenced entity.'),
    ];

    $elements['link_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link text'),
      '#default_value' => $this->getSetting('link_text'),
      '#description' => $this->t('Text to display for the popup link.'),
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $summary[] = $this->t('View mode: @view_mode', ['@view_mode' => $this->getSetting('view_mode')]);
    $summary[] = $this->t('Link text: @link_text', ['@link_text' => $this->getSetting('link_text')]);
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $view_mode = $this->getSetting('view_mode');
    $link_text = $this->getSetting('link_text');

    foreach ($items as $delta => $item) {
      $entity = $item->entity;
      if ($entity instanceof EntityInterface) {
        $entity_type = $entity->getEntityTypeId();
        $entity_id = $entity->id();
        $url = Url::fromRoute('view_entity_render_popup.modal', [
          'entity_type' => $entity_type,
          'entity_id' => $entity_id,
          'view_mode' => $view_mode,
        ]);

        $elements[$delta] = [
          '#type' => 'link',
          '#title' => $link_text,
          '#url' => $url,
          '#options' => [
            'attributes' => [
              'class' => ['use-ajax'],
              'data-dialog-type' => 'modal',
              'data-dialog-options' => json_encode(['width' => '80%']),
            ],
          ],
          '#attached' => [
            'library' => ['core/drupal.dialog.ajax'],
          ],
        ];
      }
    }

    return $elements;
  }

}